#!/usr/bin/env python
# -*- coding: utf-8 -*-

# transfert de lots de donnees cadastre MIF vers une base postgis
# 1- placer les insee.zip dans un repertoire. tous les insee.zip
#    doivent presenter la meme projection

# chemin vers le repertoire contenant les données source

PATH_SRC=""


# projection des donnees source

PROJ_SRC="EPSG:3948"


# projection des donnees destination

PROJ_DST="EPSG:2154"


# 2- creer le repertoire temporaire dans lequel 
#    les mif seront decompresses

PATH_TMP_MIF="tmp/mif"

# 3- creer si necessaire le schema de destination

PG_HOST="rbr-vgeobretagne03"
PG_PORT="5432"
PG_NAME="geobretagne"
PG_SCHEMA="pci"
PG_USER="changeme"
PG_PASSWORD="changeme"

# schema de destination
PG_SCHEMA="upload"

# droits a attribuer sur les tables creees
PG_GRANT="www-data"

# 4- a chaque fois que le script est joue, les donnees sont
#    AJOUTEES. Donc veiller a supprimer les donnees lorsque
#    l'on rejoue un departement.


import sys,os,zipfile,tarfile,shutil,logging
logging.basicConfig(format='[%(levelname)s] %(message)s', level=logging.DEBUG)

try:
    PATH_SRC=sys.argv[1]
except:
    print """
usage   : python %s src_dir
exemple : python %s ./mif
    """%(sys.argv[0],sys.argv[0])


def emptyDir(dirn):
    '''Purge a directory content'''
    files = os.listdir(dirn)
    for fn in files:
        path = os.path.join (dirn, fn)
        if os.path.isdir(path):
            emptyDir(path)
        else:
            retval = os.unlink(path)

def dirEntries(dir_name, subdir, *args):
    '''Return a list of file names found in directory 'dir_name'
    If 'subdir' is True, recursively access subdirectories under 'dir_name'.
    Additional arguments, if any, are file extensions to match filenames. Matched
        file names are added to the list.
    If there are no additional arguments, all files found in the directory are
        added to the list.
    Example usage: fileList = dirEntries(r'H:\TEMP', False, 'txt', 'py')
        Only files with 'txt' and 'py' extensions will be added to the list.
    Example usage: fileList = dirEntries(r'H:\TEMP', True)
        All files and all the files in subdirectories under H:\TEMP will be added
        to the list.
    '''
    fileList = []
    for file in os.listdir(dir_name):
        dirfile = os.path.join(dir_name, file)
        if os.path.isfile(dirfile):
            if not args:
                fileList.append(dirfile)
            else:
                if os.path.splitext(dirfile)[1][1:] in args:
                    fileList.append(dirfile)
        # recursively access file names in subdirectories
        elif os.path.isdir(dirfile) and subdir:
            fileList.extend(dirEntries(dirfile, subdir, *args))
    return fileList


def convertLot(srcPath):
    logging.info('checking %s'%srcPath)
    srcName=os.path.split(srcPath)[1]
    insee=srcName.split(".")[0].strip()
    departement=insee[0:2]
    logging.info(departement)

    # flag proceed
    proceed = True


    if proceed:

        # vidange repertoire tmp zip
        emptyDir(PATH_TMP_MIF)

        # decompression du lot
        logging.info('extracting %s into %s' % (srcPath,PATH_TMP_MIF))
        f=zipfile.ZipFile(srcPath,'r')
        f.extractall(PATH_TMP_MIF)
        f.close()

        fnMifList=dirEntries(PATH_TMP_MIF,True,"mif","MIF")

        # convertion shape
        logging.info('converting %s files to PostGIS' % len(fnMifList))
        pgparams = {
           "dbname":PG_NAME,
           "host":PG_HOST,
           "port":PG_PORT,
           "user":PG_USER,
           "password":PG_PASSWORD,
           "schema":PG_SCHEMA,
           "insee": insee,
           "departement": departement,
           "proj_src": PROJ_SRC,
           "proj_dst": PROJ_DST
        }
        for pathMif in fnMifList:
            pgparams["tablemif"]=os.path.split(pathMif)[1][:-4]
            pgparams["tablename"]=PG_SCHEMA+".cadastre_"+pgparams["tablemif"].lower()
            pgparams["src"] = pathMif
            cmd="""export PGCLIENTENCODING="iso-8859-15";ogr2ogr -append -f PostgreSQL "PG:dbname=%(dbname)s host=%(host)s port=%(port)s user=%(user)s password=%(password)s" -lco "GEOMETRY_NAME=the_geom" -lco "SPATIAL_INDEX=ON" -sql "SELECT %(departement)s AS departement, %(insee)s AS insee,* FROM %(tablemif)s" -a_srs %(proj_src)s -nln %(tablename)s -t_srs %(proj_dst)s %(src)s 2>&1"""%pgparams
            logging.info(cmd)
            os.system(cmd)




for fn in dirEntries(PATH_SRC,True,"zip","ZIP"):
    convertLot(fn)

